#ifndef IDB_H
#define IDB_H

#include <windows.h>

#define STDCOM virtual HRESULT _stdcall

#include "etype.h"

extern const GUID IID_IDB;
extern const GUID CLSID_QUICKDB;

/*
enum EType
{
	db_int,
	db_string,
	db_long,
	db_ref
};
*/

class IDB: public IUnknown
{
public:
/*************************
*     initialisation      *
**************************/
	STDCOM Init(LPCSTR name, int data_version)=0;
/*************************
*     table creation      *
**************************/
	STDCOM new_table()=0;
	STDCOM add_field(LPCSTR field_name, EType field_type, int & result)=0;
	STDCOM add_field_ref(LPCSTR field_name, LPCSTR table_name, int & result)=0;
	STDCOM commit_table(LPCSTR table_name, int & result)=0;

/*************************
*     record creation     *
**************************/
	STDCOM  new_record(LPCSTR table_name)=0;
	STDCOM  new_record(int table_number)=0;
	//operations with strings
	STDCOM  set_field(LPCSTR field_name, int field_value)=0;
	STDCOM  set_field(LPCSTR field_name, long field_value)=0;
	STDCOM  set_field(LPCSTR field_name, LPCSTR field_value)=0;
	STDCOM  set_field_ref(LPCSTR field_name, int field_value)=0;
	//operations with ints
	STDCOM  set_field(int field_number, long field_value)=0;
	STDCOM  set_field(int field_number, int field_value)=0;
	STDCOM  set_field(int field_number, LPCSTR field_value)=0;
	STDCOM  set_field_ref(int field_number, int field_value)=0;

	STDCOM  commit_record(int & result, int record_number=-1)=0;

/***********************************
*     general record operations     *
************************************/
	//operations with strings
	STDCOM  record_exist(LPCSTR table_name, int record_number, bool & result)=0;
	STDCOM  get_number_records(LPCSTR table_name, int & result)=0;
	STDCOM  delete_record(LPCSTR table_name, int record_number)=0;

	//operations with ints
	STDCOM  record_exist(int table_number, int record_number, bool & result)=0;
	STDCOM  get_number_records(int table_number, int & result)=0;
	STDCOM  delete_record(int table_number, int record_number)=0;

/***********************************
*     operations on definitions     *
************************************/
	//operations with strings
	STDCOM  get_field_type(LPCSTR table_name, LPCSTR field_name, EType & result) =0;
	STDCOM  get_field_ref_relation(LPCSTR table_name, int field_number, LPCSTR & result) =0;
	STDCOM  get_field_index(LPCSTR table_name, LPCSTR field_name, int & result) =0;
	STDCOM  get_field_name(LPCSTR table_name, int field_number, LPCSTR & result) =0;
	STDCOM  get_number_field(LPCSTR table_name, int & result)=0;

	//operations with ints
	STDCOM  get_field_type(int table_number, LPCSTR field_name, EType & result) =0;
	STDCOM  get_field_ref_relation(int table_number, int field_number, LPCSTR & result) =0;
	STDCOM  get_field_index(int table_number, LPCSTR field_name, int & result) =0;
	STDCOM  get_field_name(int table_number, int field_number, LPCSTR & result) =0;
	STDCOM  get_number_field(int table_number, int & result) =0;

/*********************************
*     record value retrieval      *
**********************************/

	//operations with strings
	STDCOM  get_field_int(LPCSTR table_name, int record_number, LPCSTR field_name, int & result)=0;
	STDCOM  get_field_long (LPCSTR table_name, int record_number, LPCSTR field_name, long & result)=0;
	STDCOM  get_field_string(LPCSTR table_name, int record_number, LPCSTR field_name, LPCSTR & result)=0;
	STDCOM  get_field_ref(LPCSTR table_name, int record_number, LPCSTR field_name, int & result)=0;

	STDCOM  get_number_children(LPCSTR table_name, int record_number, LPCSTR child_table_name, int & result)=0;
	STDCOM  get_child(LPCSTR table_name, int record_number, LPCSTR child_table_name, int index_child, int & result)=0;

	//operations with ints
	STDCOM  get_field_int(int table_number, int record_number, int field_number, int & result)=0;
	STDCOM  get_field_long(int table_number, int record_number, int field_number, long & result)=0;
	STDCOM  get_field_string(int table_number, int record_number, int field_number, LPCSTR & result)=0;
	STDCOM  get_field_ref(int table_number, int record_number, int field_number, int & result)=0;

	STDCOM  get_number_children(int table_number, int record_number, int child_table, int & result)=0;
	STDCOM  get_child(int table_number, int record_number, int child_table_number, int index_child, int & result)=0;

/**************************************
*     file operations                  *
***************************************/
	STDCOM  read_from_file(LPCSTR filename, bool & result)=0;
	STDCOM  save_to_file(LPCSTR filename, bool & result)=0;
/**************************************
*     general database operations      *
***************************************/
public:
	STDCOM  erase_data()=0;
	STDCOM  table_exist(int table_number, bool & result)=0;	
	STDCOM  get_table_index(LPCSTR table_name, int & result)=0;
	STDCOM  get_table_name(int table_index, LPCSTR & result)=0;
	STDCOM  delete_table(int table_index)=0;
	STDCOM  get_number_tables(int & result)=0; 
};

#endif